@@ -1,6 +1,6 @@
 #' Read and write chunk version data
 #'
-#' ChunkVersion data (tag 44) and ChunkVersionLegacy data (tag 118)
+#' Version data (tag 44) and LegacyVersion data (tag 118)
 #' store the version number of a chunk. In Minecraft version 1.16.100,
 #' chunk version data was moved from tag 118 to tag 44.
 #'
@@ -8,8 +8,11 @@
 NULL
 
 #' @description
-#' `get_chunk_version_data()` retrieves chunk versions from a `bedrockdb`.
-#' It will silently drop and keys not representing ChunkVersion data.
+#' `get_chunk_version_data()` and `get_chunk_version_value()` load Version
+#' data from `db`. `get_chunk_version_data()` will silently drop and keys not
+#' representing Version data. `get_chunk_version_value()` supports loading
+#' only a single value. `get_chunk_version_values()` is a synonym for
+#' `get_chunk_version_data()`.
 #'
 #' @param db A bedrockdb object.
 #' @param x,z,dimension Chunk coordinates to extract version data from.
@@ -29,13 +32,17 @@ get_chunk_version_data <- function(db, x, z, dimension) {
     purrr::map_int(dat, read_chunk_version_value)
 }
 
+#' @rdname ChunkVersion
+#' @export
+get_chunk_version_values <- get_chunk_version_data
+
 #' @rdname ChunkVersion
 #' @export
 get_chunk_version_value <- function(db, x, z, dimension) {
     key <- .process_chunk_version_key_args(x, z, dimension)
     vec_assert(key, character(), 1L)
     val <- get_value(db, key)
-    if(is.null(val)) {
+    if (is.null(val)) {
         key <- str_replace(key, ":44$", ":118")
         val <- get_value(db, key)
     }
@@ -43,19 +50,19 @@ get_chunk_version_value <- function(db, x, z, dimension) {
 }
 
 .process_chunk_version_key_args <- function(x, z, dimension) {
-    if(missing(z) && is.character(x)) {
-        # replace legacy keys with new
-        x <- str_replace(x, "(^@[^:]+:[^:]+:[^:]+):118$", "\\1:44")
+    if (missing(z) && is.character(x)) {
+        # replace legacy tags with new
+        x <- str_replace(x, ":118$", ":44")
         x <- unique(x)
     }
-    .process_key_args(x, z, dimension, tag=44L)
+    .process_key_args(x, z, dimension, tag = 44L)
 }
 
 #' @description
 #' `put_chunk_version_data()`, `put_chunk_version_values()`, and
-#' `put_chunk_version_value()` store ChunkVersion data into a `bedrockdb`.
+#' `put_chunk_version_value()` store Version data into a `bedrockdb`.
 #'
-#' @param data A named-vector of key-value pairs for ChunkVersion data.
+#' @param data A named-vector of key-value pairs for Version data.
 #'
 #' @rdname ChunkVersion
 #' @export
@@ -70,8 +77,9 @@ put_chunk_version_data <- function(db, data) {
 #' @rdname ChunkVersion
 #' @export
 put_chunk_version_values <- function(db, x, z, dimension, values) {
-    keys <- .process_key_args(x, z, dimension, tag=44L, stop_if_filtered = TRUE)
-    values <- vec_recycle(values, length(keys), x_arg="values")
+    keys <- .process_key_args(x, z, dimension, tag = 44L,
+                              stop_if_filtered = TRUE)
+    values <- vec_recycle(values, length(keys), x_arg = "values")
     values <- purrr::map(values, write_chunk_version_value)
     put_values(db, keys, values)
 }
@@ -81,21 +89,21 @@ put_chunk_version_values <- function(db, x, z, dimension, values) {
 #' @rdname ChunkVersion
 #' @export
 put_chunk_version_value <- function(db, x, z, dimension, value) {
-    key <- .process_key_args(x, z, dimension, tag=44L)
+    key <- .process_key_args(x, z, dimension, tag = 44L)
     vec_assert(key, character(), 1L)
     value <- write_chunk_version_value(value)
     put_value(db, key, value)
 }
 
 #' @description
-#' `read_chunk_version_value()` decodes ChunkVersion data.
+#' `read_chunk_version_value()` decodes Version data.
 #'
 #' @param rawdata A scalar raw.
 #'
 #' @rdname ChunkVersion
 #' @export
 read_chunk_version_value <- function(rawdata) {
-    if(is.null(rawdata)) {
+    if (is.null(rawdata)) {
         return(NULL)
     }
     vec_assert(rawdata, raw())
@@ -103,7 +111,7 @@ read_chunk_version_value <- function(rawdata) {
 }
 
 #' @description
-#' `write_chunk_version_value()` encodes ChunkVersion data.
+#' `write_chunk_version_value()` encodes Version data.
 #'
 #' @param num A scalar integer.
 #'
