@@ -62,6 +62,12 @@ nbt_string <- function(x) {
     new_nbt_string(vec_cast(x, character()))
 }
 
+#' @rdname nbt
+#' @export
+nbt_raw_string <- function(x) {
+    new_nbt_raw_string(vec_cast(x, raw()))
+}
+
 #' @rdname nbt
 #' @export
 nbt_int_array <- function(x) {
@@ -110,19 +116,19 @@ payload.default <- function(x) {
 
 #' @export
 payload.rbedrock_nbt_long <- function(x) {
-    structure(vec_data(x), class="integer64")
+    structure(vec_data(x), class = "integer64")
 }
 
 #' @export
 payload.rbedrock_nbt_long_array <- function(x) {
-    structure(vec_data(x), class="integer64")
+    structure(vec_data(x), class = "integer64")
 }
 
 #' @rdname nbt
 #' @export
 unnbt <- function(x) {
-    if(is_list(x)) {
-        rapply(x, payload, how="list")
+    if (is_list(x)) {
+        rapply(x, payload, how = "list")
     } else {
         payload(x)
     }
@@ -148,9 +154,9 @@ is_nbt_compound <- function(x) {
 
 #' @export
 `$<-.rbedrock_nbt_container` <- function(x, i, value) {
-    if(!is_nbt(value)) {
-        value <- vec_cast(value, vec_ptype(x[[i]]), x_arg="value")
-        if(!is_nbt(value)) {
+    if (!is_nbt(value)) {
+        value <- vec_cast(value, vec_ptype(x[[i]]), x_arg = "value")
+        if (!is_nbt(value)) {
             abort("conversion of value to nbt failed.")
         }
     }
@@ -159,9 +165,9 @@ is_nbt_compound <- function(x) {
 
 #' @export
 `[[<-.rbedrock_nbt_container` <- function(x, i, value) {
-    if(!is_nbt(value)) {
-        value <- vec_cast(value, vec_ptype(x[[i]]), x_arg="value")
-        if(!is_nbt(value)) {
+    if (!is_nbt(value)) {
+        value <- vec_cast(value, vec_ptype(x[[i]]), x_arg = "value")
+        if (!is_nbt(value)) {
             abort("conversion of value to nbt failed.")
         }
     }
@@ -169,8 +175,8 @@ is_nbt_compound <- function(x) {
 }
 
 .tag_assert <- function(tag, allowed) {
-    vec_assert(tag, ptype=integer(), size=1L)
-    if(!tag %in% allowed) {
+    vec_assert(tag, ptype = integer(), size = 1L)
+    if (!tag %in% allowed) {
         msg <- paste0("invalid tag `", tag, "`")
         abort(msg)
     }
@@ -204,13 +210,13 @@ new_nbt <- function(x, tag) {
 
 new_rbedrock_nbt_scalar <- function(x, class, ptype = NULL, size = NULL) {
     vec_assert(x, ptype = ptype, size = size)
-    new_vctr(x, class=c(paste0("rbedrock_nbt_", class), "rbedrock_nbt"))
+    new_vctr(x, class = c(paste0("rbedrock_nbt_", class), "rbedrock_nbt"))
 }
 
 .fixup_long <- function(x) {
     cls <- setdiff(class(x), "integer64")
     pos <- which(cls == "vctrs_vctr")
-    class(x) <- append(cls, "integer64", pos-1)
+    class(x) <- append(cls, "integer64", pos - 1)
     x
 }
 
@@ -263,6 +269,13 @@ new_nbt_byte_array <- function(x) {
 new_nbt_string <- function(x) {
     new_rbedrock_nbt_scalar(x, "string", character(), 1)
 }
+#' @keywords internal
+#' @rdname new_nbt
+#' @export
+new_nbt_raw_string <- function(x) {
+    new_rbedrock_nbt_scalar(x, "raw_string", raw())
+}
+
 #' @keywords internal
 #' @rdname new_nbt
 #' @export
@@ -274,17 +287,17 @@ new_nbt_int_array <- function(x) {
 #' @export
 new_nbt_long_array <- function(x) {
     x <- new_rbedrock_nbt_scalar(x, "long_array", bit64::integer64())
-    .fixup_long(x)   
+    .fixup_long(x)
 }
 #' @keywords internal
 #' @rdname new_nbt
 #' @export
 new_nbt_compound <- function(x) {
-    vec_assert(x, ptype=list())
-    if(!all(purrr::map_lgl(x, is_nbt))) {
+    vec_assert(x, ptype = list())
+    if (!all(purrr::map_lgl(x, is_nbt))) {
         abort("an nbt_compound can only hold nbt data")
     }
-    new_vctr(x, class=c("rbedrock_nbt_compound",
+    new_vctr(x, class = c("rbedrock_nbt_compound",
         "rbedrock_nbt", "rbedrock_nbt_container"))
 }
 #' @keywords internal
@@ -292,15 +305,16 @@ new_nbt_compound <- function(x) {
 #' @export
 new_nbt_list <- function(x) {
     ptype <- NULL
-    if(length(x) == 0) {
+    if (length(x) == 0) {
         # use a ptype of an empty list for an empty nbt_list
-        ptype=list()
+        ptype <- list()
     }
     y <- list_of(!!!x, .ptype = ptype)
     cls <- class(y)
-    structure(y, class=c("rbedrock_nbt_list", "rbedrock_nbt", cls))
+    structure(y, class = c("rbedrock_nbt_list", "rbedrock_nbt", cls))
 }
 
+# nolint start : commented_code_linter
 # new_nbt_byte_list <- function(x) {
 #     new_rbedrock_nbt_scalar(x, "byte_list", integer())
 # }
@@ -315,7 +329,7 @@ new_nbt_list <- function(x) {
 
 # new_nbt_long_list <- function(x) {
 #     x <- new_rbedrock_nbt_scalar(x, "long_list", bit64::integer64())
-#     .fixup_long(x) 
+#     .fixup_long(x)
 # }
 
 # new_nbt_float_list <- function(x) {
@@ -329,6 +343,7 @@ new_nbt_list <- function(x) {
 # new_nbt_string_list <- function(x) {
 #     new_rbedrock_nbt_scalar(x, "string_list", character())
 # }
+# nolint end
 
 ######
 
@@ -358,6 +373,8 @@ get_nbt_tag.rbedrock_nbt_byte_array <- function(x) 7L
 #' @export
 get_nbt_tag.rbedrock_nbt_string     <- function(x) 8L
 #' @export
+get_nbt_tag.rbedrock_nbt_raw_string <- function(x) 8L
+#' @export
 get_nbt_tag.rbedrock_nbt_list       <- function(x) 9L
 #' @export
 get_nbt_tag.rbedrock_nbt_compound   <- function(x) 10L
@@ -378,16 +395,18 @@ is.na.rbedrock_nbt_long_array <- function(x, ...) vec_data(NextMethod())
 
 #' Read and Write NBT Data
 #'
-#' The Named Binary Tag (NBT) format is used by Minecraft for various data types.
+#' The Named Binary Tag (NBT) format is used by Minecraft for various data
+#' types.
 #'
 #' @description
-#' `get_nbt_data()` and `get_nbt_value()` load nbt-formatted data from `db` and parses it.
-#' `get_nbt_values()` is a synonym for `get_nbt_data()`.
+#' `get_nbt_data()` and `get_nbt_value()` load nbt-formatted data from `db`
+#' and parses it. `get_nbt_values()` is a synonym for `get_nbt_data()`.
 #'
 #' @param db A `bedrockdb` object
 #' @param keys A character vector of keys.
 #' @param readoptions A `bedrock_leveldb_readoptions` object
-#' @param simplify If TRUE, simplifies a list containing a single unnamed `nbtnode`.
+#' @param simplify If TRUE, simplifies a list containing a single unnamed
+#'        `nbtnode`.
 #' @export
 get_nbt_data <- function(db, keys, readoptions = NULL, simplify = TRUE) {
     dat <- get_values(db, keys, readoptions = readoptions)
@@ -407,7 +426,8 @@ get_nbt_value <- function(db, key, readoptions = NULL, simplify = TRUE) {
 get_nbt_values <- get_nbt_data
 
 #' @description
-#' `put_nbt_values`, `put_nbt_value`, and `put_nbt_data` stores nbt data into `db` in binary form.
+#' `put_nbt_values()`, `put_nbt_value()`, and `put_nbt_data()` store nbt data
+#' into `db` in binary form.
 #'
 #' @param values A list of nbt objects
 #' @param writeoptions A `bedrock_leveldb_writeoptions` object
@@ -443,7 +463,7 @@ put_nbt_data <- function(db, data, writeoptions = NULL) {
 read_nbt <- function(rawdata, simplify = TRUE) {
     res <- read_rnbt(rawdata)
     res <- from_rnbt(res)
-    if(isTRUE(simplify) && length(res) == 1L && is.null(attributes(res))) {
+    if (isTRUE(simplify) && length(res) == 1L && is.null(attributes(res))) {
         res <- res[[1]]
     }
     res
@@ -465,7 +485,7 @@ read_nbt_data <- function(data, simplify = TRUE) {
 #' @rdname get_nbt_data
 #' @export
 write_nbt <- function(object) {
-    if(is_nbt(object)) {
+    if (is_nbt(object)) {
         object <- list(object)
     }
     object <- to_rnbt(object)
@@ -503,19 +523,36 @@ write_nbt_data <- function(data) {
 #' @export
 from_rnbt <- function(x) {
     # extract names
-    n <- purrr::map_chr(x, "name", .default="")
+    n <- purrr::map_chr(x, .extract_rnbt_name)
     # extract values
     v <- purrr::map(x, function(y) {
         from_rnbt_payload(y[["payload"]], y[["tag"]])
     })
     # Set names if any exist
-    if(all(n == "")) {
+    if (all(n == "")) {
         v
     } else {
         set_names(v, n)
     }
 }
 
+.extract_rnbt_name <- function(x) {
+    if (!is_list(x)) {
+        abort("Malformed rnbt data.")
+    }
+    n <- x$name %||% ""
+    if (is_raw(n)) {
+        # truncate at the first null
+        idx <- n == 0L
+        if (any(idx)) {
+            idx <- which(idx) - 1
+            n <- n[seq_len(idx)]
+        }
+        n <- rawToChar(n)
+    }
+    n
+}
+
 #' @rdname rnbt
 #' @keywords internal
 #' @export
@@ -523,7 +560,7 @@ to_rnbt <- function(x) {
     n <- names(x) %||% rep("", length(x))
     names(x) <- NULL
     purrr::map2(x, n, function(y, z) {
-        vec_c(list(name = z), to_rnbt_payload(y), .ptype=list())
+        vec_c(list(name = z), to_rnbt_payload(y), .ptype = list())
     })
 }
 
@@ -531,14 +568,16 @@ to_rnbt <- function(x) {
 #' @keywords internal
 #' @export
 from_rnbt_payload <- function(x, tag) {
-    if(tag == 9L) {
+    if (tag == 9L) {
         v <- purrr::map(x, function(y) {
             from_rnbt_payload(y[["payload"]], y[["tag"]])
         })
         new_nbt_list(v)
-    } else if(tag == 10L) {
+    } else if (tag == 10L) {
         ret <- from_rnbt(x)
         new_nbt_compound(ret)
+    } else if (tag == 8L && is.raw(x)) {
+        new_nbt_raw_string(x)
     } else {
         new_nbt(x, tag)
     }
@@ -596,19 +635,23 @@ format.rbedrock_nbt <- function(x, ...) {
     tag <- get_nbt_tag(x)
     suffix <- c("B", "S", "", "L", "F", "", "B", "", "", "", "", "L")
     suffix <- suffix[tag]
-    paste0(format(payload(x), nsmall=1), suffix)
+    paste0(format(payload(x), nsmall = 1), suffix)
 }
 #' @export
 format.rbedrock_nbt_string <- function(x, ...) {
     paste0("\"", format(payload(x)), "\"")
 }
 #' @export
+format.rbedrock_nbt_raw_string <- function(x, ...) {
+    format(payload(x))
+}
+#' @export
 format.rbedrock_nbt_list <- function(x, ...) {
-    lapply(payload(x),format)
+    lapply(payload(x), format)
 }
 #' @export
 format.rbedrock_nbt_compound <- function(x, ...) {
-    lapply(payload(x),format)
+    lapply(payload(x), format)
 }
 
 # ptypes -----------------------------------------------------------------------
@@ -652,24 +695,32 @@ vec_ptype2.rbedrock_nbt_float.rbedrock_nbt_float <- function(x, y, ...) x
 vec_ptype2.rbedrock_nbt_double.rbedrock_nbt_double <- function(x, y, ...) x
 
 #' @export
-vec_ptype2.rbedrock_nbt_byte_array.rbedrock_nbt_byte_array <- function(x, y, ...) x
+vec_ptype2.rbedrock_nbt_byte_array.rbedrock_nbt_byte_array <-
+    function(x, y, ...) x
 
 #' @export
 vec_ptype2.rbedrock_nbt_string.rbedrock_nbt_string <- function(x, y, ...) x
 
 #' @export
-vec_ptype2.rbedrock_nbt_compound.rbedrock_nbt_compound <- function(x, y, ...) x
+vec_ptype2.rbedrock_nbt_raw_string.rbedrock_nbt_raw_string <-
+    function(x, y, ...) x
 
 #' @export
-vec_ptype2.rbedrock_nbt_int_array.rbedrock_nbt_int_array <- function(x, y, ...) x
+vec_ptype2.rbedrock_nbt_compound.rbedrock_nbt_compound <-
+    function(x, y, ...) x
 
 #' @export
-vec_ptype2.rbedrock_nbt_long_array.rbedrock_nbt_long_array <- function(x, y, ...) x
+vec_ptype2.rbedrock_nbt_int_array.rbedrock_nbt_int_array <-
+    function(x, y, ...) x
+
+#' @export
+vec_ptype2.rbedrock_nbt_long_array.rbedrock_nbt_long_array <-
+    function(x, y, ...) x
 
 #' @export
 vec_ptype2.rbedrock_nbt_list.rbedrock_nbt_list <- function(x, y, ...) {
     x <- vec_ptype2.vctrs_list_of(x, y, ...)
-    structure(x, class=c("rbedrock_nbt_list", "rbedrock_nbt", class(x)))
+    structure(x, class = c("rbedrock_nbt_list", "rbedrock_nbt", class(x)))
 }
 
 #' @export
@@ -702,7 +753,7 @@ vec_ptype2.rbedrock_nbt_short.integer <- function(x, y, ...) integer()
 #' @export
 vec_ptype2.integer.rbedrock_nbt_short <- function(x, y, ...) integer()
 #' @export
-vec_cast.rbedrock_nbt_short.logical<- function(x, to, ...) {
+vec_cast.rbedrock_nbt_short.logical <- function(x, to, ...) {
     nbt_short(x)
 }
 #' @export
@@ -731,7 +782,7 @@ vec_ptype2.rbedrock_nbt_int.integer <- function(x, y, ...) integer()
 #' @export
 vec_ptype2.integer.rbedrock_nbt_int <- function(x, y, ...) integer()
 #' @export
-vec_cast.rbedrock_nbt_int.logical<- function(x, to, ...) {
+vec_cast.rbedrock_nbt_int.logical <- function(x, to, ...) {
     nbt_int(x)
 }
 #' @export
@@ -760,7 +811,7 @@ vec_ptype2.rbedrock_nbt_long.integer64 <- function(x, y, ...) bit64::integer64()
 #' @export
 vec_ptype2.integer64.rbedrock_nbt_long <- function(x, y, ...) bit64::integer64()
 #' @export
-vec_cast.rbedrock_nbt_long.logical<- function(x, to, ...) {
+vec_cast.rbedrock_nbt_long.logical <- function(x, to, ...) {
     nbt_long(x)
 }
 #' @export
@@ -839,7 +890,7 @@ vec_ptype2.rbedrock_nbt_byte_array.integer <- function(x, y, ...) integer()
 #' @export
 vec_ptype2.integer.rbedrock_nbt_byte_array <- function(x, y, ...) integer()
 #' @export
-vec_cast.rbedrock_nbt_byte_array.logical<- function(x, to, ...) {
+vec_cast.rbedrock_nbt_byte_array.logical <- function(x, to, ...) {
     nbt_byte_array(x)
 }
 #' @export
@@ -868,7 +919,7 @@ vec_ptype2.rbedrock_nbt_int_array.integer <- function(x, y, ...) integer()
 #' @export
 vec_ptype2.integer.rbedrock_nbt_int_array <- function(x, y, ...) integer()
 #' @export
-vec_cast.rbedrock_nbt_int_array.logical<- function(x, to, ...) {
+vec_cast.rbedrock_nbt_int_array.logical <- function(x, to, ...) {
     nbt_int_array(x)
 }
 #' @export
@@ -893,11 +944,13 @@ vec_cast.double.rbedrock_nbt_int_array <- function(x, to, ...) {
 }
 
 #' @export
-vec_ptype2.rbedrock_nbt_long_array.integer <- function(x, y, ...) bit64::integer64()
+vec_ptype2.rbedrock_nbt_long_array.integer <-
+    function(x, y, ...) bit64::integer64()
 #' @export
-vec_ptype2.integer.rbedrock_nbt_long_array <- function(x, y, ...) bit64::integer64()
+vec_ptype2.integer.rbedrock_nbt_long_array <-
+    function(x, y, ...) bit64::integer64()
 #' @export
-vec_cast.rbedrock_nbt_long_array.logical<- function(x, to, ...) {
+vec_cast.rbedrock_nbt_long_array.logical <- function(x, to, ...) {
     nbt_long_array(x)
 }
 #' @export
@@ -942,6 +995,20 @@ vec_cast.character.rbedrock_nbt_string <- function(x, to, ...) {
     vec_cast(vec_data(x), to)
 }
 
+#' @export
+vec_ptype2.rbedrock_nbt_raw_string.raw <- function(x, y, ...) raw()
+#' @export
+vec_ptype2.raw.rbedrock_nbt_raw_string <- function(x, y, ...) raw()
+#' @export
+vec_cast.rbedrock_nbt_raw_string.character <- function(x, to, ...) {
+    nbt_raw_string(x)
+}
+#' @export
+vec_cast.character.rbedrock_nbt_raw_string <- function(x, to, ...) {
+    vec_cast(vec_data(x), to)
+}
+
+
 #' @export
 vec_cast.rbedrock_nbt_list.rbedrock_nbt_list <- function(x, to, ...) {
     x <- as_list_of(x, .ptype = attr(to, "ptype"))
@@ -955,26 +1022,25 @@ vec_cast.rbedrock_nbt_list.vctrs_list_of <- function(x, to, ...) {
 #' @export
 vec_cast.rbedrock_nbt_list.double <- function(x, to, ...) {
     x <- as_list_of(as.list(x), .ptype = attr(to, "ptype"))
-    structure(x, class = class(to))    
+    structure(x, class = class(to))
 }
 #' @export
 vec_cast.rbedrock_nbt_list.integer <- function(x, to, ...) {
     x <- as_list_of(as.list(x), .ptype = attr(to, "ptype"))
-    structure(x, class = class(to))    
+    structure(x, class = class(to))
 }
 #' @export
 vec_cast.rbedrock_nbt_list.character <- function(x, to, ...) {
     x <- as_list_of(as.list(x), .ptype = attr(to, "ptype"))
-    structure(x, class = class(to))    
+    structure(x, class = class(to))
 }
 #' @export
 vec_cast.rbedrock_nbt_list.logical <- function(x, to, ...) {
     x <- as_list_of(as.list(x), .ptype = attr(to, "ptype"))
-    structure(x, class = class(to))    
+    structure(x, class = class(to))
 }
 #' @export
 vec_cast.rbedrock_nbt_list.integer64 <- function(x, to, ...) {
     x <- as_list_of(as.list(x), .ptype = attr(to, "ptype"))
-    structure(x, class = class(to))    
+    structure(x, class = class(to))
 }
-
